//*****************************************************************************************//
// Class that handles all the main plugin classes to provide a unique interrface to 	   //
// be called in mail.cpp																   //
// It will be a singleton so it can be called from callback functions so it can implement  //
// fullscreen and keyboard interactions													   //
//																				     	   //
//*****************************************************************************************//

#ifndef _GLOOBUS_
#define _GLOOBUS_

#include <dirent.h>
#include <errno.h>
#include <iostream>
#include <algorithm>

#include "gloobus-preview-plugin-manager.h"
#include "gloobus-preview-window.h"
#include "gloobus-preview-defines.h"

#include "gloobus-preview-interface-main.h"
#include "gloobus-preview-interface-image.h"
#include "gloobus-preview-interface-text.h"
#include "gloobus-preview-interface-music.h"
#include "gloobus-preview-interface-document.h"
#include "gloobus-preview-interface-video.h"
#include "gloobus-preview-interface-list.h"
#include "gloobus-preview-interface-free.h"
#include "gloobus-preview-interface-default.h"





class Gloobus 
{
 	// =========== Singleton ========= //
  	static Gloobus *m_instance;
  	Gloobus();
  	Gloobus& operator=(Gloobus&);  // Disallowed
  	Gloobus(const Gloobus&);       // Disallowed
  	
  	private:
	  	pluginManager 	m_PManager;
	  	string			m_class;
	  	string			m_plugin_name;
		vector<string>  m_files;
		string			m_filename;
		int 			m_current_file;
		gloobusWin	 *	m_gwindow;
		bool			m_fullscreen;		
		interface	 *	m_gloobus_interface;			//Polymorphism
		

	  	
	  	
	  	iImage 	  *		m_image;
	  	iImage 	  *		m_default;
	  	iMusic 	  *		m_music;
	  	iText 	  *		m_text;
	  	iDocument *		m_document;
	  	iList	  *		m_list;
	  	iMovie	  *		m_movie;
	  	iFree	  *		m_free;

	public:
  		   static Gloobus* instance		( );					//Get instance
  		   void     init			(string filename, gloobusWin *);	 
  		   void     end				( );					//PManager => Unload Plugins
  		   void 	create_plugin 	( ); 					//PManager => Search Plguin and depending on key create one or other
  		   void		destroy_plugin	( );					//Destroy plugin created
		   int		get_height		( void );				//Get plugin height
		   int 		get_width		( void );				//Get plugin width
		   void		draw_plugin		( void );
		   void		resize_window	( int x, int y );		//Resize window
		   void		fullscreen( void );				//Fullscreen

		   void		load_folder_filenames  ( void );
		   void		search_actual_filename ( void );
		   void 	next_file		( void );				//Unloads current and load next
		   void 	prev_file		( void );				//Unloads current and load prev
		   void 	end_file_and_new_file  ( void );		//Destroys the container and all its containers
		   
		   void		launch_default_application ( void );	//Gnome-open <filename> 

};


#endif
